#region

using System;
using System.ComponentModel;
using System.Drawing;
using System.Threading;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Controls.Primitives;
using System.Windows.Forms;
using System.Windows.Input;
using System.Windows.Markup;
using System.Windows.Media;
using System.Windows.Threading;
using Application = System.Windows.Application;
using MouseEventArgs = System.Windows.Forms.MouseEventArgs;

#endregion

/// Used with permission from Mariano Omar Rodriguez
/// http://weblogs.asp.net/marianor/archive/2007/10/15/a-wpf-wrapper-around-windows-form-notifyicon.aspx

namespace WPFTaskbarNotifier
{
    [ContentProperty("Text")]
    [DefaultEvent("MouseDoubleClick")]
    public class NotifyIcon : FrameworkElement, IAddChild
    {
        #region Events

        public new static readonly RoutedEvent MouseDownEvent = EventManager.RegisterRoutedEvent(
            "MouseDown", RoutingStrategy.Bubble, typeof(MouseButtonEventHandler), typeof(NotifyIcon));

        public new static readonly RoutedEvent MouseUpEvent = EventManager.RegisterRoutedEvent(
            "MouseUp", RoutingStrategy.Bubble, typeof(MouseButtonEventHandler), typeof(NotifyIcon));

        public static readonly RoutedEvent MouseClickEvent = EventManager.RegisterRoutedEvent(
            "MouseClick", RoutingStrategy.Bubble, typeof(MouseButtonEventHandler), typeof(NotifyIcon));

        public static readonly RoutedEvent MouseDoubleClickEvent = EventManager.RegisterRoutedEvent(
            "MouseDoubleClick", RoutingStrategy.Bubble, typeof(MouseButtonEventHandler), typeof(NotifyIcon));

        #endregion

        #region Dependency properties

        public static readonly DependencyProperty BalloonTipIconProperty =
            DependencyProperty.Register("BalloonTipIcon", typeof(BalloonTipIcon), typeof(NotifyIcon));

        public static readonly DependencyProperty BalloonTipTextProperty =
            DependencyProperty.Register("BalloonTipText", typeof(string), typeof(NotifyIcon));

        public static readonly DependencyProperty BalloonTipTitleProperty =
            DependencyProperty.Register("BalloonTipTitle", typeof(string), typeof(NotifyIcon));

        public static readonly DependencyProperty IconProperty =
            DependencyProperty.Register("Icon", typeof(ImageSource), typeof(NotifyIcon));

        public static readonly DependencyProperty TextProperty =
            DependencyProperty.Register("Text", typeof(string), typeof(NotifyIcon));

        #endregion

        private readonly bool initialized;
        private readonly System.Windows.Forms.NotifyIcon notifyIcon;

        public NotifyIcon()
        {
            notifyIcon = new System.Windows.Forms.NotifyIcon();
            notifyIcon.Text = Text;
            notifyIcon.Icon = FromImageSource(Icon);
            notifyIcon.Visible = FromVisibility(Visibility);

            notifyIcon.MouseDown += OnMouseDown;
            notifyIcon.MouseUp += OnMouseUp;
            notifyIcon.MouseClick += OnMouseClick;
            notifyIcon.MouseDoubleClick += OnMouseDoubleClick;

            initialized = true;
        }

        public BalloonTipIcon BalloonTipIcon
        {
            get
            {
                return (BalloonTipIcon)GetValue(BalloonTipIconProperty);
            }
            set
            {
                SetValue(BalloonTipIconProperty, value);
            }
        }

        public string BalloonTipText
        {
            get
            {
                return (string)GetValue(BalloonTipTextProperty);
            }
            set
            {
                SetValue(BalloonTipTextProperty, value);
            }
        }

        public string BalloonTipTitle
        {
            get
            {
                return (string)GetValue(BalloonTipTitleProperty);
            }
            set
            {
                SetValue(BalloonTipTitleProperty, value);
            }
        }

        public ImageSource Icon
        {
            get
            {
                return (ImageSource)GetValue(IconProperty);
            }
            set
            {
                Dispatcher.BeginInvoke(DispatcherPriority.Normal, (ThreadStart)delegate
                {
                    SetValue(IconProperty, value);
                });
            }
        }

        public string Text
        {
            get
            {
                return (string)GetValue(TextProperty);
            }
            set
            {
                SetValue(TextProperty, value);
            }
        }

        #region IAddChild Members

        void IAddChild.AddChild(object value)
        {
            throw new InvalidOperationException();
        }

        void IAddChild.AddText(string text)
        {
            if (text == null)
            {
                throw new ArgumentNullException("text");
            }
            Text = text;
        }

        #endregion

        #region Conversion members

        private static Icon FromImageSource(ImageSource icon)
        {
            if (icon == null)
            {
                return null;
            }
            Uri iconUri = new Uri(icon.ToString());
            return new Icon(Application.GetResourceStream(iconUri).Stream);
        }

        private static bool FromVisibility(Visibility visibility)
        {
            return visibility == Visibility.Visible;
        }

        private MouseButton ToMouseButton(MouseButtons button)
        {
            switch (button)
            {
                case MouseButtons.Left:
                    return MouseButton.Left;
                case MouseButtons.Right:
                    return MouseButton.Right;
                case MouseButtons.Middle:
                    return MouseButton.Middle;
                case MouseButtons.XButton1:
                    return MouseButton.XButton1;
                case MouseButtons.XButton2:
                    return MouseButton.XButton2;
            }
            throw new InvalidOperationException();
        }

        #endregion

        protected override void OnInitialized(EventArgs e)
        {
            base.OnInitialized(e);
            Dispatcher.ShutdownStarted += OnDispatcherShutdownStarted;
        }

        private void OnDispatcherShutdownStarted(object sender, EventArgs e)
        {
            notifyIcon.Dispose();
        }

        private void OnMouseDown(object sender, MouseEventArgs e)
        {
            OnRaiseEvent(MouseDownEvent, new MouseButtonEventArgs(
                                             InputManager.Current.PrimaryMouseDevice, 0, ToMouseButton(e.Button)));
        }

        private void OnMouseUp(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Right)
            {
                ShowContextMenu();
            }
            OnRaiseEvent(MouseUpEvent, new MouseButtonEventArgs(
                                           InputManager.Current.PrimaryMouseDevice, 0, ToMouseButton(e.Button)));
        }

        private void ShowContextMenu()
        {
            if (ContextMenu != null)
            {
                ContextMenuService.SetPlacement(ContextMenu, PlacementMode.MousePoint);
                ContextMenu.IsOpen = true;
            }
        }

        private void OnMouseDoubleClick(object sender, MouseEventArgs e)
        {
            OnRaiseEvent(MouseDoubleClickEvent, new MouseButtonEventArgs(
                                                    InputManager.Current.PrimaryMouseDevice, 0, ToMouseButton(e.Button)));
        }

        private void OnMouseClick(object sender, MouseEventArgs e)
        {
            OnRaiseEvent(MouseClickEvent, new MouseButtonEventArgs(
                                              InputManager.Current.PrimaryMouseDevice, 0, ToMouseButton(e.Button)));
        }

        private void OnRaiseEvent(RoutedEvent handler, MouseButtonEventArgs e)
        {
            e.RoutedEvent = handler;
            RaiseEvent(e);
        }

        protected override void OnPropertyChanged(DependencyPropertyChangedEventArgs e)
        {
            base.OnPropertyChanged(e);

            if (initialized)
            {
                switch (e.Property.Name)
                {
                    case "Icon":
                        notifyIcon.Icon = FromImageSource(Icon);
                        break;
                    case "Text":
                        notifyIcon.Text = Text;
                        break;
                    case "Visibility":
                        notifyIcon.Visible = FromVisibility(Visibility);
                        break;
                }
            }
        }

        public void ShowBalloonTip(int timeout)
        {
            notifyIcon.BalloonTipTitle = BalloonTipTitle;
            notifyIcon.BalloonTipText = BalloonTipText;
            notifyIcon.BalloonTipIcon = (ToolTipIcon)BalloonTipIcon;
            notifyIcon.ShowBalloonTip(timeout);
        }

        public void ShowBalloonTip(int timeout, string tipTitle, string tipText, BalloonTipIcon tipIcon)
        {
            notifyIcon.ShowBalloonTip(timeout, tipTitle, tipText, (ToolTipIcon)tipIcon);
        }

        public event MouseButtonEventHandler MouseClick
        {
            add
            {
                AddHandler(MouseClickEvent, value);
            }
            remove
            {
                RemoveHandler(MouseClickEvent, value);
            }
        }

        public event MouseButtonEventHandler MouseDoubleClick
        {
            add
            {
                AddHandler(MouseDoubleClickEvent, value);
            }
            remove
            {
                RemoveHandler(MouseDoubleClickEvent, value);
            }
        }

        public new event MouseButtonEventHandler MouseDown
        {
            add
            {
                AddHandler(MouseDownEvent, value);
            }
            remove
            {
                RemoveHandler(MouseDownEvent, value);
            }
        }

        public new event MouseButtonEventHandler MouseUp
        {
            add
            {
                AddHandler(MouseUpEvent, value);
            }
            remove
            {
                RemoveHandler(MouseUpEvent, value);
            }
        }
    }
}